# frozen_string_literal: true

# Marketo methods to be used in the sessions controller
module Marketo
  def submit_user_to_marketo
    add_lead(
      lead_from_form(params)
        .merge(demandbase_fields(params))
    )
  end

  # Sync this user's Munchkin profile with their main lead info
  def sync_user_with_marketo(email)
    add_lead({ email: email })
  end

  private

  def add_lead(data)
    MarketoChef.add_lead(data)
  rescue => e
    # We should never crash the application for marketing purposes
    Rails.logger.error e
  end

  def lead_from_form(params)
    {
      :FirstName                               => params[:user][:first_name],
      :LastName                                => params[:user][:last_name],
      :Email                                   => params[:user][:email],
      :Title                                   => params[:user][:job_title],
      :Company                                 => params[:user][:company],
      :Phone                                   => params[:user][:phone],
      :Describe_Your_Current_Infrastructure__c => params[:comment],
      'opscode-username'                       => params[:user][:username],
      'product-interest'                       => 'Enterprise Chef',
      :formname                                => params[:ref],
      :TOU_Accepted                            => params[:user][:terms].present? ? ?1 : ?0
    }
  end

  def demandbase_fields(params)
    return {} unless Settings.demandbase

    to_pairs = ->(k) { [k.to_sym, params[k]] }

    AccountsHelper.demandbase_field_names.map(&to_pairs).to_h
  end
end
